﻿(function () {

    "use strict";

    var ZOOM_OUT_SCALE = 10000;
    var ZOOM_IN_SCALE = 40000;
    var ZOOM_OUT_CENTER_Y = 40.6;

    // At this window width, this centerX value causes the map to be centered.
    var ZOOM_OUT_CENTER_X_WINDOW_WIDTH_BASE = 1055;
    var ZOOM_OUT_CENTER_X_BASE = 3.7;


    angular.module("enrMapModule")
           .factory("mapSvgPathBuilder", ["d3Service", "topoJsonService", function (d3Service, topoJsonService) {

               var calculateCenterX = function (windowWidth) {
                   return ZOOM_OUT_CENTER_X_BASE + (ZOOM_OUT_CENTER_X_WINDOW_WIDTH_BASE - windowWidth) / 1000;
               };

               var calculateZoomedInCenterPoint = function (zoomInMouseCoords, windowWidth) {

                   var mouseX = zoomInMouseCoords[0];
                   var mouseY = zoomInMouseCoords[1];

                   // TODO: Polish and Refactor
                   var centerX = 0.5 + (mouseX / 130);
                   centerX += (996 - windowWidth) / 1000;
                   var centerY = 42.0 - (mouseY / 185);

                   return [centerX, centerY];
               };

               var injectNewPathInfoIntoMapGeoData = function (mapGeoData, windowWidth, zoomInMouseCoords) {

                   // Geo Data by itself has no concept of how that data is presentated as a map.  Our projection determines how we interperate and draw our geo data.
                   var isZoomedIn = zoomInMouseCoords !== undefined && zoomInMouseCoords !== null;
                   var centerPoint = isZoomedIn ? calculateZoomedInCenterPoint(zoomInMouseCoords, windowWidth) : [calculateCenterX(windowWidth), ZOOM_OUT_CENTER_Y];
                   var scale = isZoomedIn ? ZOOM_IN_SCALE : ZOOM_OUT_SCALE;
                   var projection = d3Service.geo.albers()
                                             .rotate([88.3, 0, 0])
                                             .center(centerPoint)
                                             .scale(scale);

                   // Convert our geoData into SVG path data and inject that data into the mapGeoData
                   var topoJsonFeatures = topoJsonService.feature(mapGeoData, mapGeoData.objects.collection).features;
                   var mapGeometries = mapGeoData.objects.collection.geometries;
                   for (var i = 0; i < topoJsonFeatures.length; i++)
                       mapGeometries[i].pathData = d3Service.geo.path().projection(projection)(topoJsonFeatures[i]);
               };

               return {
                   injectNewPathInfoIntoMapGeoData: function (mapGeoData, windowWidth, zoomInMouseCoords) {
                       injectNewPathInfoIntoMapGeoData(mapGeoData, windowWidth, zoomInMouseCoords);
                   }
               };
           }]);

}());